// PersonTest.java
//
// Informatics 102 Spring 2012
// Code Example
//
// This class builds a Person message programmatically (using something called
// a Builder, which is an important feature of the Java code generated by the
// Protocol Buffers compiler).  After generating the message, the program writes
// it to a file, then reads it back again, printing the data within the message
// read from the file to see that it matches what we placed into it initially.

package inf102.protobuf.example;

import static inf102.protobuf.example.PersonMessage.Person;
import java.io.FileInputStream;
import java.io.FileOutputStream;


public class PersonTest
{
	public static void main(String[] args)
	throws Exception
	{
		// This is how you build a message programmatically.  Notice that you
		// first create something called a builder by calling the static
		// newBuilder() method on your generated message class.  (Note that
		// there is no space between "new" and "Builder"; we're calling a
		// static method, not a constructor here.)
		//
		// Builder objects for a particular kind of message contain setXXX()
		// methods for each of the fields in the message.  (For "repeated"
		// fields, they contain addXXX() methods instead.)  Notice, also, that
		// each of these setXXX() methods returns a copy of the Builder; this
		// is a convenience, which allows us to string together a sequence of
		// calls to the setXXX() methods, as I've done below.
		//
		// Once all fields are set, we call the build() method to actually
		// build our Person message, which returns a Person object.  Notably,
		// objects of our message classes like Person are immutable, which is
		// why the builders are necessary.
		
		Person alex = Person.newBuilder()
			.setName("Alex Thornton")
			.setTitle("Continuing Lecturer")
			.build();


		// Here, we write the message to a file called "persontest.dat".
		// Notice that we're using a FileOutputStream here instead of a
		// FileWriter; this is because we're writing raw bytes, as opposed
		// to text.  (In general, throughout the Java library, this is the
		// distinction between readers and writers, on the one hand, and
		// streams on the other.  Readers and writes read and write text,
		// i.e., Unicode characters.  Streams read and write bytes.)

		FileOutputStream outputFile = new FileOutputStream("persontest.dat");
		alex.writeTo(outputFile);
		outputFile.close();


		// Now we open the same file and read a Person from it.  The static
		// method Person.parseFrom() is used; we pass it an input stream or
		// a byte[] (in this case, the FileInputStream) and it reads the
		// bytes from that stream and builds us a message.
		
		FileInputStream inputFile = new FileInputStream("persontest.dat");
		Person p = Person.parseFrom(inputFile);
		inputFile.close();


		// Finally, to verify that we got back what we started with, we
		// print out the name and the title of our Person.
		
		System.out.println(p.getName());
		System.out.println(p.getTitle());
	}
}
